from brownie import Token, accounts

def main():
    token = Token.deploy("{{token_name}}", {{token_symbol}}}, {{decimal_count}}, 1e23, {'from': accounts[0]})
    return token

def distribute_tokens(sender=accounts[0], receiver_list=accounts[1:]):
    token = main()
    for receiver in receiver_list:
        token.transfer(receiver, 1e18, {'from': sender})
